@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://git.sr.ht/~protesilaos/fontaine
 ;; Mailing list: https://lists.sr.ht/~protesilaos/fontaine
-;; Version: 0.1.0
+;; Version: 0.1.1
 ;; Package-Requires: ((emacs "27.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -52,7 +52,9 @@
 ;;      :italic-slant italic
 ;;      :line-spacing 1)
 ;;
-;; The doc string of `fontaine-presets' explains all properties in detail.
+;; The doc string of `fontaine-presets' explains all properties in detail
+;; and documents some important caveats or information about font settings
+;; in Emacs.
 ;;
 ;; The command `fontaine-set-preset' applies the desired preset.  If there
 ;; is only one available, it implements it outright.  Otherwise it produces
@@ -64,7 +66,11 @@
 ;;
 ;; The command `fontaine-set-face-font' prompts with completion for a face
 ;; and then asks the user to specify the value of the relevant properties.
-;; The list of faces to choose from is the same as that implied by the
+;; Preferred font families can be defined in the user option
+;; `fontaine-font-families', otherwise Fontaine will try to find suitable
+;; options among the fonts installed on the system (not always reliable,
+;; depending on the Emacs build and environment it runs in).  The list of
+;; faces to choose from is the same as that implied by the
 ;; `fontaine-presets'.  Properties to change and their respective values
 ;; will depend on the face.  For example, the `default' face requires a
 ;; natural number for its height attribute, whereas every other face needs
@@ -72,6 +78,17 @@
 ;; command is for interactive use only and is supposed to be used for
 ;; previewing certain styles before eventually codifying them as presets.
 ;;
+;; Changing the `bold' and `italic' faces only has a noticeable effect if
+;; the underlying theme does not hardcode a weight and slant but inherits
+;; from those faces instead (e.g. the `modus-themes').
+;;
+;; The latest value of `fontaine-set-preset' is stored in a file whose
+;; location is defined in `fontaine-latest-state-file'.  Saving is done by
+;; the `fontaine-store-latest-preset' function, which should be assigned to
+;; a hook (e.g. `kill-emacs-hook').  To restore that value, the user can
+;; call the function `fontaine-restore-latest-preset' (such as by adding it
+;; to their init file).
+;;
 ;; As for the name of this package, it is the French word for "fountain"
 ;; which, in turn, is what the font or source is.  However, I will not
 ;; blame you if you can only interpret it as a descriptive acronym: FONTs
@@ -179,18 +196,28 @@ The properties in detail:
 - The `:line-spacing' specifies the value of the `line-spacing'
   variable.
 
-Note that all the properties for `bold' and `italic' will only
-have a noticeable effect if the active theme does not hardcode a
-weight and a slant, but instead inherits the relevant face (such
-as the `modus-themes').
+Use the desired preset with the command `fontaine-set-preset'.
 
-Also note that a height attribute for anything other than the
-`default' face must be set to a floating point, which is
-understood as a multiple of the default height (this allows all
-faces to scale harmoniously).  The `:default-height' always is a
-natural number.
+Caveats or further notes:
 
-Use the desired preset with the command `fontaine-set-preset'."
+- On a Windows system, setting a `default' weight other than
+  `regular' or `normal' will not work.  This is a limitation with
+  Emacs on that system.
+
+- All the properties for `bold' and `italic' will only have a
+  noticeable effect if the active theme does not hardcode a
+  weight and a slant, but instead inherits the relevant
+  face (such as the `modus-themes').
+
+- A height attribute for anything other than the `default' face
+  must be set to a floating point, which is understood as a
+  multiple of the default height (this allows all faces to scale
+  harmoniously).  The `:default-height' always is a natural
+  number.
+
+- Fontaine does not [yet] support Emacs' fontsets for other
+  scripts or character sets (e.g. Emoji).  Read the documentation
+  in the Info node `(emacs) Modifying Fontsets'."
   :group 'fontaine
   :type `(alist
           :value-type
@@ -222,6 +249,53 @@ Use the desired preset with the command `fontaine-set-preset'."
                   ((const :tag "Line spacing" :line-spacing) ,(get 'line-spacing 'custom-type))))
           :key-type symbol))
 
+(defcustom fontaine-latest-state-file
+  (locate-user-emacs-file "fontaine-latest-state.eld")
+  "File to save the latest value of `fontaine-set-preset'.
+Saving is done by the `fontaine-store-latest-preset' function,
+which should be assigned to a hook (e.g. `kill-emacs-hook').
+
+This is then used to restore the last value with the function
+`fontaine-restore-latest-preset'."
+  :type 'file
+  :group 'fontaine)
+
+(defcustom fontaine-font-families nil
+  "An alist of preferred font families.
+
+The expected value of this option is a triplet of cons cells
+where the car is `default', `fixed-pitch', or `variable-pitch'
+and the cdr is a list of strings that reference font family
+names.  For example:
+
+    (setq fontaine-font-families
+          \\='((default \"Iosevka Comfy\" \"Hack\" \"Roboto Mono\")
+            (fixed-pitch \"Mononoki\" \"Source Code Pro\" \"Fira Code\")
+            (variable-pitch \"Noto Sans\" \"Roboto\" \"FiraGO\")))
+
+
+This is used at the minibuffer prompt while using the command
+`fontaine-set-face-font' to prompt for a font family.  When this
+user option is nil, that prompt will try to find all relevant
+fonts installed on the system, which might not always be
+reliable (depending on the Emacs build and the environment it
+runs in).
+
+If only the `default' is nil and the others are specified, the
+command `fontaine-set-face-font' will produce results that
+combine the other two lists."
+  :type '(set
+          (cons :tag "Default font families"
+                (const default)
+                (repeat string))
+          (cons :tag "Fixed pitch font families"
+                (const fixed-pitch)
+                (repeat string))
+          (cons :tag "Variable pitch font families"
+                (const variable-pitch)
+                (repeat string)))
+  :group 'fontaine)
+
 ;;;; General utilities
 
 (defun fontaine--set-face-attributes (face family &optional weight height)
@@ -233,7 +307,7 @@ Use the desired preset with the command `fontaine-set-preset'."
     ;; ;; Hence why the following fails.  Keeping it for posterity...
     ;; (set-face-attribute face nil :family family :weight weight :height height)
     (if (eq (face-attribute face :weight) weight)
-          (internal-set-lisp-face-attribute face :family family 0)
+        (internal-set-lisp-face-attribute face :family family 0)
       (internal-set-lisp-face-attribute face :weight weight 0)
       (internal-set-lisp-face-attribute face :family family 0)
       (internal-set-lisp-face-attribute face :weight weight 0))
@@ -340,6 +414,7 @@ outright, else prompt with completion."
     (fontaine--apply-bold-preset preset)
     (fontaine--apply-italic-preset preset)
     (setq fontaine--current-preset preset)
+    (add-to-history 'fontaine--preset-history (format "%s" preset))
     (run-hooks 'fontaine-set-preset-hook)))
 
 ;;;; Modify individual faces
@@ -360,8 +435,12 @@ outright, else prompt with completion."
 (defvar fontaine--face-history '()
   "Minibuffer history of `fontaine-set-face-font'.")
 
-(defvar fontaine--font-family-history '()
-  "Minibuffer history of selected font families.")
+(define-obsolete-variable-alias
+  'fontaine--font-family-history
+  'fontaine--default-font-family-history "2022-04-29")
+
+(defvar fontaine--default-font-family-history '()
+  "Minibuffer history of selected `default' font families.")
 
 ;; We have `font-family-list', which is unfiltered.
 (defun fontaine--family-list-fixed-pitch (&optional frame)
@@ -382,6 +461,9 @@ Target FRAME, if provided as an optional argument."
      ;; certain families.
      (x-family-fonts nil frame)))))
 
+;; NOTE 2022-04-29: This is known to not work on Windows, except for
+;; Emacs 29.  Read:
+;; <https://lists.gnu.org/archive/html/emacs-devel/2022-04/msg01281.html>.
 (defun fontaine--family-list-variable-pitch (&optional frame)
   "Return a list of available proportionately spaced font families.
 Target FRAME, if provided as an optional argument."
@@ -399,44 +481,59 @@ Target FRAME, if provided as an optional argument."
 
 (defun fontaine--set-default ()
   "Set `default' attributes interactively."
-  (let ((family (completing-read "Font family of `default' face: "
-                                 (font-family-list) nil t
-                                 nil 'fontaine--font-family-history))
-        (weight (intern (completing-read "Select weight for `default': "
-                                         fontaine--font-weights nil)))
-        (height (read-number "Height of `default' face (must be a natural number): "
-                             nil 'fontaine--natnum-history)))
+  (let* ((families (or (alist-get 'default fontaine-font-families)
+                       (append (alist-get 'fixed-pitch fontaine-font-families)
+                               (alist-get 'variable-pitch fontaine-font-families))
+                       (font-family-list)))
+         (family (completing-read "Font family of `default' face: "
+                                  families nil t
+                                  nil 'fontaine--default-font-family-history))
+         (weight (intern (completing-read "Select weight for `default': "
+                                          fontaine--font-weights nil)))
+         (height (read-number "Height of `default' face (must be a natural number): "
+                              (or (string-to-number (nth 0 fontaine--natnum-history)))
+                              'fontaine--natnum-history)))
     (if (natnump height)
-        (fontaine--set-face-attributes 'default family weight)
+        (fontaine--set-face-attributes 'default family weight height)
       (user-error "Height of `default' face must be a natural number"))))
 
 (defvar fontaine--float-history '()
   "Minibuffer history for floating point numbers.")
 
+(defvar fontaine--fixed-pitch-font-family-history '()
+  "Minibuffer history of selected `fixed-pitch' font families.")
+
 (defun fontaine--set-fixed-pitch ()
   "Set `fixed-pitch' attributes interactively."
-  (let ((family (completing-read "Font family of `fixed-pitch': "
-                                 (fontaine--family-list-fixed-pitch) nil t
-                                 nil 'fontaine--font-family-history))
-        (weight (intern (completing-read "Select weight for `fixed-pitch': "
-                                         fontaine--font-weights nil)))
-        (height (read-number "Height of `fixed-pitch' face (must be a floating point): "
-                             1.0 'fontaine--float-history)))
+  (let* ((families (or (alist-get 'fixed-pitch fontaine-font-families)
+                       (fontaine--family-list-fixed-pitch)))
+         (family (completing-read "Font family of `fixed-pitch': "
+                                  families nil t nil
+                                  'fontaine--fixed-pitch-font-family-history))
+         (weight (intern (completing-read "Select weight for `fixed-pitch': "
+                                          fontaine--font-weights nil)))
+         (height (read-number "Height of `fixed-pitch' face (must be a floating point): "
+                              1.0 'fontaine--float-history)))
     (if (floatp height)
-        (fontaine--set-face-attributes 'fixed-pitch family weight)
+        (fontaine--set-face-attributes 'fixed-pitch family weight height)
       (user-error "Height of `fixed-pitch' face must be a floating point"))))
 
+(defvar fontaine--variable-pitch-font-family-history '()
+  "Minibuffer history of selected `variable-pitch' font families.")
+
 (defun fontaine--set-variable-pitch ()
   "Set `variable-pitch' attributes interactively."
-  (let ((family (completing-read "Font family of `variable-pitch': "
-                                 (fontaine--family-list-variable-pitch) nil t
-                                 nil 'fontaine--font-family-history))
-        (weight (intern (completing-read "Select weight for `variable-pitch': "
-                                         fontaine--font-weights nil)))
-        (height (read-number "Height of `variable-pitch' face (must be a floating point): "
-                             1.0 'fontaine--float-history)))
+  (let* ((families (or (alist-get 'variable-pitch fontaine-font-families)
+                       (fontaine--family-list-variable-pitch)))
+         (family (completing-read "Font family of `variable-pitch': "
+                                  families nil t nil
+                                  'fontaine--variable-pitch-font-family-history))
+         (weight (intern (completing-read "Select weight for `variable-pitch': "
+                                          fontaine--font-weights nil)))
+         (height (read-number "Height of `variable-pitch' face (must be a floating point): "
+                              1.0 'fontaine--float-history)))
     (if (floatp height)
-        (fontaine--set-face-attributes 'variable-pitch family weight)
+        (fontaine--set-face-attributes 'variable-pitch family weight height)
       (user-error "Height of `variable-pitch' face must be a floating point"))))
 
 (defun fontaine--set-bold ()
@@ -453,13 +550,28 @@ Target FRAME, if provided as an optional argument."
 
 ;;;###autoload
 (defun fontaine-set-face-font (face)
-  "Set font and other attributes of FACE.
-If FACE is not part of `fontaine--font-faces', fall back to
-calling `fontaine-set-preset'.
+  "Set font and/or other attributes of FACE.
 
-Changing the `bold' and `italic' faces only has a noticeable
-effect if the underlying does not hardcode a weight and slant but
-inherits from those faces instead (e.g. the `modus-themes')."
+When called interactively, prompt for FACE and then continue
+prompting for the relevant face attributes each of which depends
+on the FACE (for example, the `default' FACE accepts a family, a
+height as a natural number, and a weight, whereas `bold' only
+accepts a weight).
+
+With regard to the font family that some faces accept, the
+candidates are those specified in the user option
+`fontaine-font-families'.  If none are specified, try to find
+relevant installed fonts and provide them as completion
+candidates.
+
+Note that changing the `bold' and `italic' faces only has a
+noticeable effect if the underlying does not hardcode a weight
+and slant but inherits from those faces instead (e.g. the
+`modus-themes').
+
+When called from Lisp (albeit discouraged), if FACE is not part
+of `fontaine--font-faces', fall back to interactively calling
+`fontaine-set-preset'."
   (declare (interactive-only t))
   (interactive
    (list
@@ -475,6 +587,40 @@ inherits from those faces instead (e.g. the `modus-themes')."
     ('variable-pitch (fontaine--set-variable-pitch))
     (_ (call-interactively #'fontaine-set-preset))))
 
+;;;; Store and restore preset
+
+(defvar fontaine--preset-history '()
+  "Minibuffer history of preset configurations.")
+
+;;;###autoload
+(defun fontaine-store-latest-preset ()
+  "Write latest cursor state to `fontaine-latest-state-file'.
+Can be assigned to `kill-emacs-hook'."
+  (when-let ((hist fontaine--preset-history))
+    (with-temp-file fontaine-latest-state-file
+      (insert ";; Auto-generated file; don't edit -*- mode: "
+              (if (<= 28 emacs-major-version)
+                  "lisp-data"
+                "emacs-lisp")
+              " -*-\n")
+      (pp (intern (car hist)) (current-buffer)))))
+
+(defvar fontaine-recovered-preset nil
+  "Recovered value of latest store cursor preset.")
+
+;;;###autoload
+(defun fontaine-restore-latest-preset ()
+  "Restore latest preset set by `fontaine-set-preset'.
+The value is stored in `fontaine-latest-state-file'."
+  (when-let* ((file fontaine-latest-state-file)
+              ((file-exists-p file)))
+    (setq fontaine-recovered-preset
+          (unless (zerop
+                   (or (file-attribute-size (file-attributes file))
+                       0))
+            (with-temp-buffer
+              (insert-file-contents file)
+              (read (current-buffer)))))))
 
 (provide 'fontaine)
 ;;; fontaine.el ends here
